/*
 *   Copyright            : (C) 2003 by Jakob Schroeter
 *   License              : This file is released under the terms of the LGPL, version 2.
 *   Email                : js@camaya.de
 */


#ifndef _KONSCREENDISPLAY_H
#define _KONSCREENDISPLAY_H

#include <qwidget.h>
#include <qcolor.h>
#include <qstring.h>
#include <qpixmap.h>
#include <qtimer.h>
#include <qlayout.h>

class KOnScreenDisplayPrivate;

/**
* This class implements an On Screen Display. It shows a textual message
* and optionally an icon on top of all windows at a configurable location.
* It does not get the focus and therefore does not interrupt the user's work.
*
* @author Jakob Schrter (js@camaya.de)
* @since 3.3
*/
class KOnScreenDisplay : public QWidget
{
    Q_OBJECT

public:
    enum OsdMoveDir {
      ToBottom = 0,
      ToTop
    };

    KOnScreenDisplay( QWidget *parent, const char *name = 0 );
    virtual ~KOnScreenDisplay();
    /**
     * Convenience function. Creates and displays an On Screen Display based on the settings in kosdrc.
     * @param text The text that will be displayed.
     * @return Returns a pointer to the On Screen Display. You should connect a slot to the signalEnd to be notified at the end of the OSD's life.
     */
    static KOnScreenDisplay *message( QWidget *parent, const QString &text );

    /**
     * Convenience function. Creates and displays an On Screen Display based on the settings in kosdrc
     * and the parameters found in the config parameter.
     * Shows an icon left of the text.
     * @param text The text that will be displayed.
     * @param icon An pixmap of the icon to be displayed
     * @param config An unsigned int containing the (QRgb) value of the foreground color
     * If specified, this option overwrites the default color configuration.
     * @return Returns a pointer to the On Screen Display. You should connect a slot to the signalEnd to be notified at the end of the OSD's life.
     */
    static KOnScreenDisplay *message( QWidget *parent, const QString &text, const QPixmap &icon, const unsigned int color );

    /**
     * Sets the position of the text.
     * @param position The position of the text.
     * @param verticalOff The vertical offset from the position. Default is 0.
     * @param horizontalOff The horizontal offset from the position. Default is 0.
     */
    virtual void setPosition( const int position, const int verticalOff = 0, const int horizontalOff = 0 );

    /**
     * Sets the time the text will be displayed in seconds.
     * @param timeout The number of seconds the text will be displayed.
     */
    virtual void setTimeout( const int timeout );

    /**
     * Sets the foreground (text) color.
     */
    virtual void setFgColor( const QColor &fgColor );

    /**
     * Sets the background color. Only used when the background is not transparent.
     */
    virtual void setBgColor( const QColor &bgColor );

    /**
     * Sets the text to be displayed. You can use Qt rich text tags here.
     * Leading "<qt><nobr>" and trailing "</qt></nobr>" are hard coded.
     */
    virtual void setText( const QString &text );

    /**
     * Switches the shadow on or off, sets the shadow size.
     * @param shadow Switch shadow on or off.
     * @param offset The size of the shadow in pixels. Default is 1.
     */
    virtual void setShadow( const bool shadow, const int offset = 1 );

    /**
     * Sets the background to transparent or solid.
     */
    virtual void setTransparency( const bool transparency );

    /**
     * Enables OSD scrolling and sets the scroll speed
     */
    virtual void setScrolling( const bool scroll, const int scrollSpeed = 0 );

    /**
     * sets the OSD's moving direction (for both scrolling and jumping)
     */
    virtual void setMoving( const int moveDir );

    /**
     * Enables OSD blinking.
     * @param time Number of milliseconds the osd will blink.
     * @param freq A half blinking period in milliseconds.
     */
    virtual void setBlinking( const int time, const int freq );

    /**
     * Enables OSD jumping. To use this feature it is necessary to connect a signal to the slotJump().
     * This signal should be emitted right before a new On Screen Display is created.
     */
    virtual void setJumping( const bool jump );

    /*
     * Sets an icon (or any pixmap) to be displayed next to the text and a position of the icon.
     * @param position QtAlignmentFlags. Left, Top, Bottom and RighT are valid values.
     * @param spacer Sets the amount of space between text and icon
     */
    virtual void setIcon( const QPixmap icon, const int position = Qt::AlignLeft, const int spacer = 5 );

    /*
     * Set whether to deleteLater() or not.
     */
    virtual void setAutoDelete( const bool autoDelete );

    /**
     * Creates and displays the OSD. Should be the last step. Calling this function
     * is not necessary when the static @ref message() functions are used.
     * Additionally it starts the autohide (set with @ref setTimeout()) timer.
     */
    virtual void show();

    /**
     * Loads the config.
     */
    void loadConfig();

public slots:
    void slotJump();
    void slotRefreshOSD();

signals:
    /**
     * This signal is emitted when the On Screen Display times out (aka is finally hidden) and can be deleted (or deletes itself).
     */
    void signalEnd();

protected slots:
    void slotBlinkTimeout();
    void slotTimeout();
    void slotScroll();
    void slotBlink();

protected:
    void paintEvent( QPaintEvent* );

private:
    void positionSelf();
    void repaint();
    void create();
    void createTextOSD();
    void createWidgetOSD();
    int getNewYPosition( int oldPos, const int step );
    QColor m_fgColor;
    QColor m_bgColor;
    QPixmap m_bgpic;
    QPixmap m_icon;
    QString m_barPrefix;
    QString m_text;
    QWidget *m_parent;
    QWidget *m_widget;
    QTimer *scrollTimer;
    QTimer *blinkTimer;
    QTimer *fadeTimer;
    QVBoxLayout *m_layout;
    int m_moveDir;
    int m_barType;
    int m_horizontalOff;
    int m_verticalOff;
    int m_iconPosition;
    int m_iconSpacer;
    int m_shadowOffset;
    int m_scrollSpeed;
    int m_barPercent;
    int m_barWidth;
    int m_barHeight;
    int m_blinkTime;
    int m_blinkFreq;
    int m_position;
    int m_timeout;
    int m_parentWidth;
    int m_parentHeight;
    bool m_configChanged;
    bool m_transparency;
    bool m_autoDelete;
    bool m_shadow;
    bool m_scroll;
    bool m_fade;
    bool m_jump;

    KOnScreenDisplayPrivate *d;
};

#endif

